require 'faraday'
require 'active_attr'
module Fme
  module Api
    class Request
      include ActiveAttr::Model
      attr_accessor :host, :jwt, :debug

      def self.protocol
        if Rails.env.development?
          'http://'
        else
          'https://'
        end
      end

      def http_cache_options
        http_cache_options = {
          shared_cache: false,
          store: Rails.cache
        }
        if debug
          http_cache_options[:logger] = Rails.logger
        end
        http_cache_options
      end

      def healthy?
        connection.get(service_url('/okcomputer/all.json')).body.dig('default', 'success')
      end

      private

      def connection
        @connection ||= Faraday.new(url: "#{self.class.protocol}#{host}") do |builder|
          setup_connection(builder)
        end
      end

      def setup_connection(builder)
        builder.options[:open_timeout] = 2
        builder.options[:timeout] = 5
        builder.authorization :Bearer, jwt
        builder.request :json
        builder.request :url_encoded
        builder.request :retry, max: 5, interval: 0.05, interval_randomness: 0.5, backoff_factor: 2
        builder.use FaradayCacheServiceDown, Rails.cache
        builder.use :http_cache, http_cache_options
        builder.use FaradayFollowNextLinks, 5
        builder.response :json, content_type: /\bjson$/
        builder.response :raise_error
        builder.response :logger if debug
        builder.adapter Faraday.default_adapter
      end
    end
  end
end
